from typing import List, Callable, TypeVar, Union
from time import time_ns
from datetime import datetime
import random
from .types import TimeStampedMsg, CssColorType, RgbColor
from pathlib import Path


def ClassProps(cls) -> List[str]:
    return [i for i in cls.__dict__.keys() if i[:2] != '__']


def ClassConsts(cls) -> List[str]:
    return list(map(lambda d: cls.__dict__[d], ClassProps(cls)))


def time_s() -> float:
    '''
    returns the current time in seconds since epoche
    '''
    return (time_ns() // 1000000) / 1000.0


def without_none(raw: dict) -> dict:
    '''
    removes all None values from the dict on level 1 (thus nested dicts are not supported). 
    '''
    return {k: v for k, v in raw.items() if v is not None}


def flatten(list_of_lists: List[List]) -> List:
    return [y for x in list_of_lists for y in x]


def to_datetime(data: Union[TimeStampedMsg, int, float]) -> datetime:
    '''
    extracts the datetime from a data package. if the field `time_stamp` is not present,
    the current datetime will be returned
    '''
    is_number = isinstance(data, int) or isinstance(data, float)

    if not is_number and 'time_stamp' not in data:
        return datetime.now()

    ts = data if (isinstance(data, int) or isinstance(data, float)) else data['time_stamp']
    # convert time_stamp from ms to seconds
    if ts > 1000000000000:
        ts = ts / 1000.0
    return datetime.fromtimestamp(ts)


def to_css_color(color: CssColorType, base_color: RgbColor = (255, 0, 0)) -> str:
    if isinstance(color, str):
        if len(color) > 1:
            return color
        elif len(color) == 1:
            color = int(color, 10)
        else:
            color = 0
    if isinstance(color, int):
        if base_color is None or len(base_color) < 3:
            base_color = (255, 0, 0)

        return f'rgba({base_color[0]},{base_color[1]},{base_color[2]},{color / 9})'

    if len(color) == 3:
        return f'rgb({color[0]},{color[1]},{color[2]})'
    elif len(color) > 3:
        return f'rgba({color[0]},{color[1]},{color[2]},{color[3]})'
    else:
        raise AttributeError(color)


def random_color() -> str:
    '''
    Returns
    -------
    str
        random rgb color, colors ranging between 0 and 255

    Example
    -------
        rgb(127, 1, 36)
    '''
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)

    return f'rgb({r}, {g}, {b})'


def try_or(func, default=None, expected_exc=(Exception,)):
    try:
        return func()
    except expected_exc:
        return default


T = TypeVar('T')


def first(filter_func: Callable[[T], bool], list_: List[T]) -> Union[T, None]:
    return next((item for item in list_ if try_or(lambda: filter_func(item), False)), None)


def image_to_lines(image: str):
    lines = image.splitlines(False)
    without_empty = list(filter(lambda l: len(l.strip()) > 0, lines))
    return list(map(lambda line: line.strip(), without_empty))


def lines_to_grid(lines: List[str]):
    return list(map(lambda line: [*line], lines))



class DictX(dict):
    '''
    dict with the ability to access keys over dot notation,
    e.g.

    ```py
    data = DictX({
        "foo": "bar"
    })

    print(data.foo)     # use dot to get
    data.foo = 'blaa'   # use dot to assign
    del data.foo        # use dot to delete
    ```
    credits: https://dev.to/0xbf/use-dot-syntax-to-access-dictionary-key-python-tips-10ec
    '''

    def __getitem__(self, key):
        try:
            val = dict.__getitem__(self, key)
            if type(val) is dict:
                return DictX(val)
            return val
        except KeyError as k:
            # deepcopy does not work propperly when no AttributeError is raised here!!!
            raise AttributeError(k)

    def __getattr__(self, key):
        try:
            if type(self[key]) is dict:
                return DictX(self[key])
            return self[key]
        except KeyError as k:
            pass

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        try:
            del self[key]
        except KeyError as k:
            raise AttributeError(k)

    def __repr__(self):
        return '<DictX ' + dict.__repr__(self) + '>'


if __name__ == '__main__':
    a = DictX({'a': DictX({'b': 12, 'c': {'a': 113}})})
    a['b'] = {'c': 18}
    print(a['b'])
    print(a.b)
    print(a['a'])
    print(a.a)
    print(a.a.b)
    print(a['a']['c'])
    print(a.a.c.a)



from __future__ import annotations
from typing import overload, Union, Literal, Optional, Tuple, List, TypedDict
from dataclasses import dataclass
from .dictx import DictX
from enum import Enum

Number = Union[float, int]


class InputType(str, Enum):
    TEXT = 'text'
    NUMBER = 'number'
    DATETIME = 'datetime'
    DATE = 'date'
    TIME = 'time'
    SELECT = 'select'


class SocketEvents(str, Enum):
    ALL_DATA = "all_data"
    CLEAR_DATA = "clear_data"
    DATA_STORE = "data_store"
    DEVICE = "device"
    DEVICES = "devices"
    ERROR_MSG = "error_msg"
    GET_ALL_DATA = "get_all_data"
    GET_DEVICES = "get_devices"
    INFORMATION_MSG = "information_msg"
    JOIN_ROOM = "join_room"
    LEAVE_ROOM = "leave_room"
    NEW_DATA = "new_data"
    NEW_DEVICE = "new_device"
    REMOVE_ALL = "remove_all"
    ROOM_JOINED = "room_joined"
    ROOM_LEFT = "room_left"
    SET_NEW_DEVICE_NR = "set_new_device_nr"


class DataType(str, Enum):
    ACCELERATION = "acceleration"
    ALERT_CONFIRM = "alert_confirm"
    ALL_DATA = "all_data"
    BORDER_OVERLAP = "border_overlap"
    CLEAR_PLAYGROUND = "clear_playground"
    COLOR = "color"
    GRID = "grid"
    GRID_UPDATE = "grid_update"
    GYRO = "gyro"
    INPUT_PROMPT = "input_prompt"
    INPUT_RESPONSE = "input_response"
    KEY = "key"
    NOTIFICATION = "notification"
    PLAYGROUND_CONFIG = "playground_config"
    POINTER = "pointer"
    REMOVE_SPRITE = "remove_sprite"
    SPRITE = "sprite"
    SPRITES = "sprites"
    SPRITE_CLICKED = "sprite_clicked"
    SPRITE_COLLISION = "sprite_collision"
    SPRITE_OUT = "sprite_out"
    SPRITE_REMOVED = "sprite_removed"
    REMOVE_LINE = "remove_line"
    LINE = "line"
    LINES = "lines"
    UNKNOWN = "unknown"


class ClientDataMsgInputType(str, Enum):
    DATE = "date"
    DATETIME_LOCAL = "datetime-local"
    NUMBER = "number"
    SELECT = "select"
    TEXT = "text"
    TIME = "time"


R = int
G = int
B = int
A = float

RgbColor = Union[Tuple[R, G, B], List[int]]
CssColorType = Union[int, str, RgbColor, Tuple[R, G, B, A], List[float], List[Union[int, float]]]
BaseColor = Union[str, RgbColor]


@dataclass
class DeliveryOptions:
    broadcast: bool = False
    unicast_to: Optional[int] = None
    device_id: Optional[str] = None


@dataclass
class TimeStampedMsg(DictX):
    time_stamp: Optional[Union[float, int]]


class BaseMsg(TimeStampedMsg):
    device_id: Optional[str]
    device_nr: Optional[int]


class DataMsg(BaseMsg):
    type: DataType


class Device(DictX):
    device_id: str
    is_client: bool
    device_nr: int
    socket_id: str


class DevicesMsg(TimeStampedMsg):
    devices: List[Device]


class DeviceJoinedMsg(BaseMsg):
    room: str
    device: Device


class DeviceLeftMsg(BaseMsg):
    room: str
    device: Device


class Key(DataMsg):
    type: Literal['key']
    key: Literal['up', 'right', 'down', 'left', 'home', 'F1', 'F2', 'F3', 'F4']


class KeyMsg(Key):
    time_stamp: float
    device_id: str
    device_nr: int


@overload
def default(type: Literal['key']) -> KeyMsg: ...
@overload
def default(type: Literal['acceleration']) -> AccMsg: ...
@overload
def default(type: Literal['gyro']) -> GyroMsg: ...
@overload
def default(type: Literal['grid_pointer']) -> GridPointerMsg: ...
@overload
def default(type: Literal['color_pointer']) -> ColorPointerMsg: ...
@overload
def default(type: Literal['notification']) -> Notification: ...
@overload
def default(type: Literal['pointer']) -> PointerDataMsg: ...
@overload
def default(type: Literal['border_overlap']) -> BorderOverlapMsg: ...
@overload
def default(type: Literal['sprite_clicked']) -> SpriteClickedMsg: ...
@overload
def default(type: Literal['sprite_collision']) -> SpriteCollisionMsg: ...
@overload
def default(type: Literal['sprite_out']) -> SpriteOut: ...


def default(type: str) -> DictX:
    base = {'time_stamp': 0,
            'device_id': '',
            'device_nr': -999}
    if type == 'key':
        return DictX({
            **base,
            'type': 'key',
            'key': ''
        })
    elif type == 'acceleration':
        return DictX({
            **base,
            'x': 0,
            'y': 0,
            'z': 0,
            'interval': 16
        })
    elif type == 'gyro':
        return DictX({
            **base,
            'alpha': 0,
            'beta': 0,
            'gamma': 0,
            'absolute': False
        })
    elif type == 'color_pointer':
        return DictX({
            **base,
            'type': 'pointer',
            'context': 'color',
            'x': 0,
            'y': 0,
            'width': -1,
            'height': -1,
            'displayed_at': 0
        })
    elif type == 'grid_pointer':
        return DictX({
            **base,
            'type': 'pointer',
            'context': 'grid',
            'x': 0,
            'y': 0,
            'width': -1,
            'height': -1,
            'displayed_at': 0
        })
    elif type == 'pointer':
        return default('color_pointer')
    elif type == 'border_overlap':
        return DictX({
            **base,
            'type': 'border_overlap',
            'border': 'left',
            'collision_detection': False,
            'x': 0,
            'y': 0,
            'id': ''
        })
    elif type == 'sprite_clicked':
        return DictX({
            **base,
            'type': 'sprite_clicked',
            'id': '',
            'text': '',
            'x': 0,
            'y': 0
        })
    elif type == 'sprite_collision':
        return DictX({
            **base,
            'type': 'sprite_collision',
            'sprites': ['s1', 's2'],
            'overlap': 'in'
        })
    elif type == 'sprite_out':
        return DictX({
            **base,
            'type': 'sprite_out',
            'id': ''
        })
    elif type == 'notification':
        return DictX({
            **base,
            'type': 'notification',
            'message': '',
            'alter': False,
            'time': 3000
        })
    return DictX({})


def default_data_frame():
    return DictX({
                'key': default('key'),
                'acceleration': default('acceleration'),
                'gyro': default('gyro'),
                'color_pointer': default('color_pointer'),
                'grid_pointer': default('grid_pointer'),
                'pointer': default('pointer'),
                'border_overlap': default('border_overlap'),
                'sprite_clicked': default('sprite_clicked'),
                'sprite_collision': default('sprite_collision'),
                'sprite_out': default('sprite_out')
            })


class KeyMsgF1(KeyMsg):
    key: Literal['F1']


class KeyMsgF2(KeyMsg):
    key: Literal['F2']


class KeyMsgF3(KeyMsg):
    key: Literal['F3']


class KeyMsgF4(KeyMsg):
    key: Literal['F4']


class PointerData(DataMsg):
    type: Literal['pointer']
    context: Literal['color', 'grid']


class PointerDataMsg(PointerData):
    time_stamp: float
    device_id: str
    device_nr: int


class ColorPointer(PointerDataMsg):
    context: Literal['color']
    x: int
    y: int
    width: int
    height: int
    displayed_at: float


class ColorPointerMsg(ColorPointer):
    time_stamp: float
    device_id: str
    device_nr: int


class GridPointer(PointerDataMsg):
    context: Literal['grid']
    row: int
    column: int
    number: int
    color: str
    displayed_at: float


class GridPointerMsg(GridPointer):
    time_stamp: float
    device_id: str
    device_nr: int


class Acc(BaseMsg):
    type: Literal['acceleration']
    x: float
    y: float
    z: float
    interval: float


class AccMsg(Acc):
    time_stamp: float
    device_id: str
    device_nr: int


class Gyro(BaseMsg):
    type: Literal['gyro']
    alpha: float
    beta: float
    gamma: float
    absolute: bool


class GyroMsg(Gyro):
    time_stamp: float
    device_id: str
    device_nr: int


class DataFrame(DictX):
    key: KeyMsg
    acceleration: AccMsg
    gyro: GyroMsg
    color_pointer: ColorPointerMsg
    grid_pointer: GridPointerMsg
    pointer: PointerDataMsg
    border_overlap: BorderOverlapMsg
    sprite_clicked: SpriteClickedMsg
    sprite_collision: SpriteCollisionMsg
    sprite_out: SpriteOutMsg


class ErrorMsg(BaseMsg):
    type: SocketEvents
    msg: str
    err: Union[str, dict]


class InformationMsg(TimeStampedMsg):
    message: str
    action: DataMsg
    should_retry: Optional[bool]


class InputResponse(DataMsg):
    type: Literal['input_response']
    response: str
    displayed_at: float


class InputResponseMsg(InputResponse):
    time_stamp: float
    device_id: str
    device_nr: int


class AlertConfirm(DataMsg):
    type: Literal['alert_confirm']
    displayed_at: float


class AlertConfirmMsg(AlertConfirm):
    time_stamp: float
    device_id: str
    device_nr: int


ColorGrid = Union[CssColorType, List[str], List[CssColorType], List[List[CssColorType]]]


class Grid(DataMsg):
    type: Literal['grid']
    grid: ColorGrid
    base_color: RgbColor


class GridMsg(Grid):
    time_stamp: float
    device_id: str
    device_nr: int


class Notification(DataMsg):
    type: Literal['notification']
    message: str
    alter: Optional[bool]
    time: Optional[Union[int, float]]


class PlaygroundConfig(DataMsg):
    type: Literal['playground_config']
    width: Optional[int]
    height: Optional[int]
    shift_x: Optional[int]
    shift_y: Optional[int]
    color: Optional[str]


class PlaygroundConfigMsg(PlaygroundConfig):
    time_stamp: float
    device_id: str
    device_nr: int


class SpriteForm(str, Enum):
    RECTANGLE = "rectangle"
    ROUND = "round"


class Sprite(DataMsg):
    id: str
    clickable: Optional[bool] = None
    collision_detection: Optional[bool] = None
    color: Optional[str] = None
    border_color: Optional[str] = None
    direction: Optional[List[float]] = None
    distance: Optional[float] = None
    form: Optional[SpriteForm] = None
    height: Optional[float] = None
    pos_x: Optional[float] = None
    pos_y: Optional[float] = None
    reset_time: Optional[bool] = None
    speed: Optional[float] = None
    text: Optional[str] = None
    font_size: Optional[Number] = None
    font_color: Optional[str] = None
    time_span: Optional[float] = None
    width: Optional[float] = None


class SpriteMsg(Sprite):
    type: Literal['sprite']
    time_stamp: float
    device_id: str
    device_nr: int


class UpdateSprite(DataMsg):
    type: Literal['sprite']
    pos_x: int
    pos_y: int
    width: int
    height: int
    form: Literal['round', 'rectangle']
    color: CssColorType


class UpdateSpriteMsg(UpdateSprite):
    time_stamp: float
    device_id: str
    device_nr: int


class Line(DataMsg):
    id: str
    y1: Number
    x1: Number
    x2: Number
    y2: Number
    color: Optional[str]
    line_width: Optional[Number]
    rotate: Optional[Number]
    anchor: Optional[Number]


class LineMsg(Line):
    type: Literal['line']
    time_stamp: float
    device_id: str
    device_nr: int


class UpdateLine(DataMsg):
    x1: Optional[Number]
    y1: Optional[Number]
    x2: Optional[Number]
    y2: Optional[Number]
    color: Optional[str]
    line_width: Optional[Number]
    rotate: Optional[Number]
    anchor: Optional[Number]


class UpdateLineMsg(UpdateLine):
    type: Literal['line']
    time_stamp: float
    device_id: str
    device_nr: int


class SpriteBase:
    id: str
    collision_detection: bool


class SpriteCollision(DataMsg):
    type: Literal['sprite_collision']
    sprites: Tuple[Union[Sprite, SpriteBase], Union[Sprite, SpriteBase]]
    time_stamp: float
    overlap: Literal['in', 'out']


class SpriteCollisionMsg(SpriteCollision):
    time_stamp: float
    device_id: str
    device_nr: int


class BorderOverlap(DataMsg):
    type: Literal['border_overlap']
    border: Literal['left', 'right', 'top', 'bottom']
    collision_detection: bool
    x: float
    y: float
    id: str
    sprite: Optional[Sprite]


class BorderOverlapMsg(BorderOverlap):
    time_stamp: float
    device_id: str
    device_nr: int


class SpriteClicked(DataMsg):
    type: Literal['sprite_clicked']
    id: str
    text: Optional[str]
    x: Number
    y: Number
    sprite: Optional[Sprite]


class SpriteClickedMsg(SpriteClicked):
    time_stamp: float
    device_id: str
    device_nr: int


class SpriteOut(DataMsg):
    type: Literal['sprite_out']
    id: str
    sprite: Optional[Sprite]


class SpriteOutMsg(SpriteOut):
    time_stamp: float
    device_id: str
    device_nr: int


class SpriteRemoved(DataMsg):
    type: Literal['sprite_removed']
    id: str
    sprite: Optional[Sprite]


class SpriteRemovedMsg(SpriteRemoved):
    time_stamp: float
    device_id: str
    device_nr: int


class Unknown(DictX):
    type: Literal['unknown']


class UnknownMsg(Unknown):
    time_stamp: float
    device_id: str
    device_nr: int


ClientMsg = Union[
    KeyMsg,
    GridMsg,
    AccMsg,
    GyroMsg,
    PointerDataMsg,
    InputResponseMsg,
    AlertConfirmMsg,
    SpriteMsg,
    UpdateSpriteMsg,
    PlaygroundConfigMsg,
    SpriteOutMsg,
    SpriteCollisionMsg,
    UnknownMsg
]

ClientData = Union[
    Key,
    Grid,
    Acc,
    Gyro,
    PointerData,
    InputResponse,
    AlertConfirm,
    Sprite,
    UpdateSprite,
    PlaygroundConfig,
    SpriteOut,
    SpriteCollision,
    Unknown,
    Notification
]



from enum import Enum
from random import sample


class Colors(str, Enum):
    ALICEBLUE = "aliceblue"
    ANTIQUEWHITE = "antiquewhite"
    AQUA = "aqua"
    AQUAMARINE = "aquamarine"
    AZURE = "azure"
    BEIGE = "beige"
    BISQUE = "bisque"
    BLACK = "black"
    BLANCHEDALMOND = "blanchedalmond"
    BLUE = "blue"
    BLUEVIOLET = "blueviolet"
    BROWN = "brown"
    BURLYWOOD = "burlywood"
    CADETBLUE = "cadetblue"
    CHARTREUSE = "chartreuse"
    CHOCOLATE = "chocolate"
    CORAL = "coral"
    CORNFLOWERBLUE = "cornflowerblue"
    CORNSILK = "cornsilk"
    CRIMSON = "crimson"
    CYAN = "cyan"
    DARKBLUE = "darkblue"
    DARKCYAN = "darkcyan"
    DARKGOLDENROD = "darkgoldenrod"
    DARKGRAY = "darkgray"
    DARKGREEN = "darkgreen"
    DARKKHAKI = "darkkhaki"
    DARKMAGENTA = "darkmagenta"
    DARKOLIVEGREEN = "darkolivegreen"
    DARKORANGE = "darkorange"
    DARKORCHID = "darkorchid"
    DARKRED = "darkred"
    DARKSALMON = "darksalmon"
    DARKSEAGREEN = "darkseagreen"
    DARKSLATEBLUE = "darkslateblue"
    DARKSLATEGRAY = "darkslategray"
    DARKTURQUOISE = "darkturquoise"
    DARKVIOLET = "darkviolet"
    DEEPPINK = "deeppink"
    DEEPSKYBLUE = "deepskyblue"
    DIMGRAY = "dimgray"
    DODGERBLUE = "dodgerblue"
    FIREBRICK = "firebrick"
    FLORALWHITE = "floralwhite"
    FORESTGREEN = "forestgreen"
    FUCHSIA = "fuchsia"
    GAINSBORO = "gainsboro"
    GHOSTWHITE = "ghostwhite"
    GOLD = "gold"
    GOLDENROD = "goldenrod"
    GRAY = "gray"
    GREEN = "green"
    GREENYELLOW = "greenyellow"
    HONEYDEW = "honeydew"
    HOTPINK = "hotpink"
    INDIANRED = "indianred"
    INDIGO = "indigo"
    IVORY = "ivory"
    KHAKI = "khaki"
    LAVENDER = "lavender"
    LAVENDERBLUSH = "lavenderblush"
    LAWNGREEN = "lawngreen"
    LEMONCHIFFON = "lemonchiffon"
    LIGHTBLUE = "lightblue"
    LIGHTCORAL = "lightcoral"
    LIGHTCYAN = "lightcyan"
    LIGHTGOLDENRODYELLOW = "lightgoldenrodyellow"
    LIGHTGREEN = "lightgreen"
    LIGHTGREY = "lightgrey"
    LIGHTPINK = "lightpink"
    LIGHTSALMON = "lightsalmon"
    LIGHTSEAGREEN = "lightseagreen"
    LIGHTSKYBLUE = "lightskyblue"
    LIGHTSLATEGRAY = "lightslategray"
    LIGHTSTEELBLUE = "lightsteelblue"
    LIGHTYELLOW = "lightyellow"
    LIME = "lime"
    LIMEGREEN = "limegreen"
    LINEN = "linen"
    MAGENTA = "magenta"
    MAROON = "maroon"
    MEDIUMAQUAMARINE = "mediumaquamarine"
    MEDIUMBLUE = "mediumblue"
    MEDIUMORCHID = "mediumorchid"
    MEDIUMPURPLE = "mediumpurple"
    MEDIUMSEAGREEN = "mediumseagreen"
    MEDIUMSLATEBLUE = "mediumslateblue"
    MEDIUMSPRINGGREEN = "mediumspringgreen"
    MEDIUMTURQUOISE = "mediumturquoise"
    MEDIUMVIOLETRED = "mediumvioletred"
    MIDNIGHTBLUE = "midnightblue"
    MINTCREAM = "mintcream"
    MISTYROSE = "mistyrose"
    MOCCASIN = "moccasin"
    NAVAJOWHITE = "navajowhite"
    NAVY = "navy"
    OLDLACE = "oldlace"
    OLIVE = "olive"
    OLIVEDRAB = "olivedrab"
    ORANGE = "orange"
    ORANGERED = "orangered"
    ORCHID = "orchid"
    PALEGOLDENROD = "palegoldenrod"
    PALEGREEN = "palegreen"
    PALETURQUOISE = "paleturquoise"
    PALEVIOLETRED = "palevioletred"
    PAPAYAWHIP = "papayawhip"
    PEACHPUFF = "peachpuff"
    PERU = "peru"
    PINK = "pink"
    PLUM = "plum"
    POWDERBLUE = "powderblue"
    PURPLE = "purple"
    REBECCAPURPLE = "rebeccapurple"
    RED = "red"
    ROSYBROWN = "rosybrown"
    ROYALBLUE = "royalblue"
    SADDLEBROWN = "saddlebrown"
    SALMON = "salmon"
    SANDYBROWN = "sandybrown"
    SEAGREEN = "seagreen"
    SEASHELL = "seashell"
    SIENNA = "sienna"
    SILVER = "silver"
    SKYBLUE = "skyblue"
    SLATEBLUE = "slateblue"
    SLATEGRAY = "slategray"
    SNOW = "snow"
    SPRINGGREEN = "springgreen"
    STEELBLUE = "steelblue"
    TAN = "tan"
    TEAL = "teal"
    THISTLE = "thistle"
    TOMATO = "tomato"
    TURQUOISE = "turquoise"
    VIOLET = "violet"
    WHEAT = "wheat"
    WHITE = "white"
    WHITESMOKE = "whitesmoke"
    YELLOW = "yellow"
    YELLOWGREEN = "yellowgreen"

    def __repr__(self):
        return self._value_

    def __str__(self):
        return f'{self._value_}'

    __iter_idx__ = 0
    __shuffle_idx__ = 0
    __shuffled__ = sample(range(141), 141)

    @classmethod
    def next(cls):
        cls.__iter_idx__ = (int(cls.__iter_idx__) + 1) % len(cls)
        return list(cls)[cls.__iter_idx__ - 1]

    @classmethod
    def random(cls):
        cls.__shuffle_idx__ = (int(cls.__shuffle_idx__) + 1) % 141
        return list(cls)[int(cls.__shuffled__[cls.__shuffle_idx__ - 1])]


if __name__ == '__main__':
    print(Colors.next())
    print(Colors.random())
